using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;


namespace LowerCensor
{
    public class Censor : MVRScript
    {
		// Game Objects and components
		public FreeControllerV3 CUIAtomControl;
		public Atom person;
		public FreeControllerV3 rThigh;
		public FreeControllerV3 lThigh;
		protected Transform lowerCensor;
		protected RectTransform lowerRect;

		public FreeControllerV3 rNip;
		public FreeControllerV3 lNip;
		protected Transform upperCensor;
		protected RectTransform upperRect;

		public JSONStorableFloat upperCensorSize;		
		public JSONStorableFloat upperCensorRelativeDistance;
		public JSONStorableFloat upperVerticalAdjust;

		public JSONStorableFloat lowerCensorSize;		
		public JSONStorableFloat lowerCensorRelativeDistance;
		public JSONStorableFloat lowerVerticalAdjust;
		
		public List<string> ptAChoices;
		public Dictionary<string, ForceReceiver> receiverNameToForceReceiver;

		protected Canvas dynCanvas;
		protected Canvas dynCanvas2;
		public override void Init()
        {			 
            try
            {
				person = containingAtom;
				rNip = person.freeControllers.First(fc => fc.name == "rNippleControl");
				lNip = person.freeControllers.First(fc => fc.name == "lNippleControl");
				rThigh = person.freeControllers.First(fc => fc.name == "rThighControl");
				lThigh = person.freeControllers.First(fc => fc.name == "lThighControl");

				GameObject g = new GameObject();
				dynCanvas = g.AddComponent<Canvas>();
				dynCanvas.renderMode = RenderMode.WorldSpace;
				CanvasScaler cs = g.AddComponent<CanvasScaler>();
				cs.scaleFactor = 100.0f;
				cs.dynamicPixelsPerUnit = 1f;
				GraphicRaycaster gr = g.AddComponent<GraphicRaycaster>();
				lowerRect = g.GetComponent<RectTransform>();
				lowerRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, 500f);
				lowerRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, 100f);
				g.transform.localScale = new Vector3(0.0006f, 0.0006f, 0.0006f);
				g.transform.localPosition = new Vector3(.15f, .2f, .7f);
				lowerCensor = g.transform;

				Image image = g.AddComponent<Image>();
    			image.color = new Color(0.0F, 0.0F, 0.0F);

				g = new GameObject();
				dynCanvas2 = g.AddComponent<Canvas>();
				dynCanvas2.renderMode = RenderMode.WorldSpace;
				cs = g.AddComponent<CanvasScaler>();
				cs.scaleFactor = 100.0f;
				cs.dynamicPixelsPerUnit = 1f;
				gr = g.AddComponent<GraphicRaycaster>();
				upperRect = g.GetComponent<RectTransform>();
				upperRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, 500f);
				upperRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, 100f);
				g.transform.localScale = new Vector3(0.0006f, 0.0006f, 0.0006f);
				g.transform.localPosition = new Vector3(.15f, .2f, .7f);
				upperCensor = g.transform;

				image = g.AddComponent<Image>();
    			image.color = new Color(0.0F, 0.0F, 0.0F);

				upperCensorSize = new JSONStorableFloat("Upper Censor Size", 1.0f, setUpperSize, 0.0f, 10.0f, false);
                RegisterFloat(upperCensorSize);
                CreateSlider(upperCensorSize);

				upperCensorRelativeDistance = new JSONStorableFloat("Upper Censor Distance", 0.2f, 0.0f, 10.0f, false);
                RegisterFloat(upperCensorRelativeDistance);
                CreateSlider(upperCensorRelativeDistance);

				upperVerticalAdjust = new JSONStorableFloat("Upper Vertical Adjust", 0.0f, -1.0f, 1.0f, false);
                RegisterFloat(upperVerticalAdjust);
                CreateSlider(upperVerticalAdjust);

				lowerCensorSize = new JSONStorableFloat("Lower Censor Size", 1.0f, setLowerSize, 0.0f, 10.0f, false);
                RegisterFloat(lowerCensorSize);
                CreateSlider(lowerCensorSize);

				lowerCensorRelativeDistance = new JSONStorableFloat("Lower Censor Distance", 0.2f, 0.0f, 10.0f, false);
                RegisterFloat(lowerCensorRelativeDistance);
                CreateSlider(lowerCensorRelativeDistance);

				lowerVerticalAdjust = new JSONStorableFloat("Lower Vertical Adjust", -0.08f, -1.0f, 1.0f, false);
                RegisterFloat(lowerVerticalAdjust);
                CreateSlider(lowerVerticalAdjust);
				
			}
            catch(Exception e)
            {
                SuperController.LogError("inBetween - Exception caught: " + e);
            }
		}
		
		private void setUpperSize(float f) {
			upperRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, f * 500f);
			upperRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, f * 100f);
		}

		private void setLowerSize(float f) {
			lowerRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, f * 500f);
			lowerRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, f * 100f);
		}

		private void Start() {
			CUIAtomControl = containingAtom.GetStorableByID("control") as FreeControllerV3;
		}


		void Update() {
			{
				Vector3 adjustedPos =  (rThigh.transform.position + lThigh.transform.position) / 2 + new Vector3(0, lowerVerticalAdjust.val, 0);
				Vector3 pathVector = adjustedPos - CameraTarget.centerTarget.transform.position;
				Vector3 unitVector = pathVector.normalized;
				// SuperController.LogMessage("Vectors: " + pathVector + " " + unitVector);

				lowerCensor.position = adjustedPos - unitVector * lowerCensorRelativeDistance.val;
				Vector3 lookPos = CameraTarget.centerTarget.transform.position - lowerCensor.position;
				Quaternion lookRot = Quaternion.LookRotation(lookPos, Vector3.up);
				float eulerX = 0;
				//if (CUIRotateConstrainToY.val == false)
				//{
				eulerX = lookRot.eulerAngles.x;
				//}
				float eulerY = lookRot.eulerAngles.y;
				Quaternion rotation = Quaternion.Euler (eulerX, eulerY, 0);
				
				lowerCensor.rotation = rotation;
			}
			{
				float a = Vector3.Angle(rNip.transform.position, lNip.transform.position);
				// SuperController.LogMessage("" + rNip.transform.position. + " " + lNip.transform.position + " " + a);
				Vector3 adjustedPos = (rNip.transform.position + lNip.transform.position) / 2 + new Vector3(0, upperVerticalAdjust.val, 0);
				Vector3 pathVector = adjustedPos - CameraTarget.centerTarget.transform.position;
				Vector3 unitVector = pathVector.normalized;
				// SuperController.LogMessage("Vectors: " + pathVector + " " + unitVector);

				upperCensor.position = adjustedPos - unitVector * upperCensorRelativeDistance.val;
				Vector3 lookPos = CameraTarget.centerTarget.transform.position - upperCensor.position;
				Quaternion lookRot = Quaternion.LookRotation(lookPos, Vector3.up);
				float eulerX = 0;
				//if (CUIRotateConstrainToY.val == false)
				//{
				eulerX = lookRot.eulerAngles.x;
				//}
				float eulerY = lookRot.eulerAngles.y;
				Quaternion rotation = Quaternion.Euler (eulerX, eulerY, 0);
				
				upperCensor.rotation = rotation;
			}
		}
		
		void OnDestroy() {
			if (dynCanvas != null) {
				//SuperController.singleton.RemoveCanvas(dynCanvas);
				Destroy(dynCanvas.gameObject);
				Destroy(dynCanvas2.gameObject);
			}
		}
	}
}